cdef extern from "Python.h":
    void Py_INCREF(object)
    void Py_DECREF(object)

cdef extern from "stdlib.h":
    void* malloc(int size)          
    void free(void* ptr)    

cdef extern from "stdio.h":
    int fopen(char *filename, char *mode)
    int fclose(int stream)
 
cdef extern from "xing_header.h":
    void xingheader2str(void *ctx, char *buf)
    
                
cdef extern from "playif.h":
    void * mpgedit_play_init(char *name)
    int  mpgedit_play_frame(void *ctx) except *
    int  mpgedit_play_seek_time(void *ctx, long sec, long msec)
    long mpgedit_play_total_size(void *ctx)
    long mpgedit_play_total_sec(void *ctx)
    long mpgedit_play_total_msec(void *ctx)
    long mpgedit_play_current_sec(void *ctx)
    long mpgedit_play_current_msec(void *ctx)
    void mpgedit_play_close(void *ctx)
    ctypedef int (*status)(void *user_data, long sec, long msec) except *
    void mpgedit_play_set_status_callback(void *ctx, status callback, void *user_data)

cdef extern from "volumeif.h":
    void * mpgedit_volume_open()
    void   mpgedit_volume_close(void *ctx)
    int    mpgedit_volume_get(void *ctx, int *lvol, int *rvol)
    void   mpgedit_volume_set(void *ctx, int lvol, int rvol)
            
cdef extern from "editif.h":
    #
    # edit_index
    #
    void * mpgedit_edit_index_init(char *mp3file)   
    int  mpgedit_edit_index(void *ctx) except *
    void mpgedit_edit_index_free(void *ctx)
    long mpgedit_edit_index_sec(void *ctx)
    long mpgedit_edit_index_msec(void *ctx)
    long mpgedit_edit_index_frames(void *ctx)
    long mpgedit_edit_index_offset(void *ctx)

    #
    # mp3time
    #        
    struct _mpeg_time:
        long units
        long usec   
        
    #
    # editspec
    #       
    ctypedef struct editspec_t
            
    editspec_t *mpgedit_editspec_init()
    void       mpgedit_editspec_free(editspec_t *ctx)
    void       mpgedit_editspec_append(editspec_t *ctx, char *mp3file, char *timespec)
    char      *mpgedit_editspec_get_file(editspec_t *ctx, int offset)
    long       mpgedit_editspec_get_length(editspec_t *ctx)
    _mpeg_time *mpgedit_editspec_get_stime(editspec_t *ctx, int offset)
    _mpeg_time *mpgedit_editspec_get_etime(editspec_t *ctx, int offset)
   

    #
    # edit_files
    #               
    ctypedef struct mpgedit_t
    
    mpgedit_t * mpgedit_edit_files_init(editspec_t *editspec_ctx, char *outfile, unsigned int flags, int *rstatus)
    void   mpgedit_edit_files_free(mpgedit_t *ctx)
    int    mpgedit_edit_files(mpgedit_t *ctx, int *rstatus)
    long   mpgedit_edit_frames(mpgedit_t *ctx)
    long   mpgedit_edit_sec(mpgedit_t *ctx)
    long   mpgedit_edit_offset(mpgedit_t *ctx)
    int    mpgedit_edit_has_xing(mpgedit_t *ctx)
    void * mpgedit_edit_xing_header(mpgedit_t *ctx)
   

class MpgeditError(Exception): pass    

default_callback = eval('lambda *args: 1')
                   
cdef class Play:   
    cdef void   *ctx
    cdef object  callback
    cdef void   *volume_ctx
    
    def __new__(self, mp3file):   
        self.ctx = mpgedit_play_init(mp3file)
        self.volume_ctx = mpgedit_volume_open()

    def __dealloc__(self):
        self.close()

    def close(self):
        if self.volume_ctx:                        
            mpgedit_volume_close(self.volume_ctx)    
            self.volume_ctx = NULL
            
        if self.ctx:            
            mpgedit_play_close(self.ctx)
            self.ctx = NULL        
            
    def play(self, callback=None):
        if not callback:
            callback = default_callback        
        
        self.callback = callback                                
        mpgedit_play_set_status_callback(self.ctx, _callback_wrapper, <void*>callback)                                    
        while(mpgedit_play_frame(self.ctx)): pass            
                                                                
    def seek_time(self, sec, msec):
        return mpgedit_play_seek_time(self.ctx, sec, msec)     
        
    def total_size(self):
        return mpgedit_play_total_size(self.ctx)    
            
    def total_time(self):
        return mpgedit_play_total_sec(self.ctx), mpgedit_play_total_msec(self.ctx)     

    def total_info(self):
        return mpgedit_play_total_size(self.ctx), \
               (mpgedit_play_total_sec(self.ctx), mpgedit_play_total_msec(self.ctx))
                    
    def current_time(self):
        return mpgedit_play_current_sec(self.ctx), mpgedit_play_current_msec(self.ctx)

    def get_volume(self):
        cdef int lvol
        cdef int rvol
        cdef int status
                
        status = mpgedit_volume_get(self.volume_ctx, &lvol, &rvol)
        if status == -1:
            raise IndexError('Unable to retrieve volume information')        
        return lvol, rvol
        
    def set_volume(self, left, right=-1):
        if right == -1: right = left
        mpgedit_volume_set(self.volume_ctx, left, right)      

        
        
cdef int _callback_wrapper(void *callback, long sec, long msec) except *:      
    return (<object>callback)(sec, msec)
    

cdef class Index:
    cdef void *ctx
    
    def __new__(self, mp3file):
        self.ctx = mpgedit_edit_index_init(mp3file)  

    def __dealloc__(self):
        self.close()

    def close(self):
        if self.ctx:
            mpgedit_edit_index_free(self.ctx);                            
            self.ctx = NULL
                                                           
    def index(self, callback=None):
        if not callback:
            callback = default_callback

        while(not mpgedit_edit_index(self.ctx)): 
            if not callback(mpgedit_edit_index_frames(self.ctx), 
                            mpgedit_edit_index_sec(self.ctx), mpgedit_edit_index_msec(self.ctx),
                            mpgedit_edit_index_offset(self.ctx)): break
                            
    def frames(self):
        return mpgedit_edit_index_frames(self.ctx)
        
    def time(self):
        return mpgedit_edit_index_sec(self.ctx), mpgedit_edit_index_msec(self.ctx)        

    def offset(self):
        return mpgedit_edit_index_offset(self.ctx)

    def info(self):
        return (mpgedit_edit_index_frames(self.ctx), 
                (mpgedit_edit_index_sec(self.ctx), mpgedit_edit_index_msec(self.ctx)), 
                mpgedit_edit_index_offset(self.ctx))
                            
cdef class Spec:
    cdef editspec_t *ctx
        
    def __new__(self):        
        self.ctx = mpgedit_editspec_init()
                
    def __dealloc__(self):            
        self.close()

    def close(self):
        if self.ctx:
            mpgedit_editspec_free(self.ctx)
            self.ctx = NULL
                    
    def append(self, mp3file, timespec):
        mpgedit_editspec_append(self.ctx, mp3file, timespec)

    def filename(self, offset):
        cdef char *file
        file = mpgedit_editspec_get_file(self.ctx, index)               
        if not file: raise IndexError('offset is out of range')
        return file
        
    def length(self):
        return mpgedit_editspec_get_length(self.ctx)
        
    def stime(self, offset):
        cdef _mpeg_time *stime
        stime = mpgedit_editspec_get_stime(self.ctx, offset)        
        if not stime: raise IndexError('offset is out of range')                
        sunits = stime.units
        susec = stime.usec
        return sunits, susec

    def etime(self, offset):
        cdef _mpeg_time *etime
        etime = mpgedit_editspec_get_etime(self.ctx, offset)                
        if not etime: raise IndexError('offset is out of range')                
        eunits = etime.units
        eusec = etime.usec        
        return eunits, eusec                        
                           
    def info(self, offset):
        cdef char       *file
        cdef _mpeg_time *stime
        cdef _mpeg_time *etime
        
        file = mpgedit_editspec_get_file(self.ctx, offset)                
        if not file: raise IndexError('list index out of range')        
        
        stime = mpgedit_editspec_get_stime(self.ctx, offset)        
        if not stime: raise IndexError('offset is out of range')                        
        sunits = stime.units
        susec = stime.usec        
        
        etime = mpgedit_editspec_get_stime(self.ctx, offset)              
        if not etime: raise IndexError('offset is out of range')                       
        eunits = etime.units
        eusec = etime.usec        
        
        return file, (sunits, susec), (eunits, eusec)
        
    def info_all(self):
        all = []
        i = 0
        try:
            while(1):
                all.append(self.info(i))
                i = i + 1
        except IndexError:
            return all
        
edit_errors = {1: 'Unable to append to output file',
               2: 'Output file exists',
               3: 'Unable to read input file',
               4: 'Unable to read index file',
               5: 'Unable to open output file',
               6: 'Invalid Edit object'}
                    
cdef class Edit:
    cdef mpgedit_t *ctx
    
    def __new__(self, Spec spec, outfile, flags=0):
        cdef int rstatus
        self.ctx = mpgedit_edit_files_init(spec.ctx, outfile, flags, &rstatus)
        if self.ctx == NULL and rstatus != 0:            
            raise MpgeditError(rstatus, edit_errors[rstatus])        
        
    def __dealloc__(self):        
        self.close()

    def close(self):
        if self.ctx:
            mpgedit_edit_files_free(self.ctx)
            self.ctx = NULL
                
    def edit(self, callback=None):
        cdef int rstatus                

        if not callback:
            callback = default_callback
                               
        while(mpgedit_edit_files(self.ctx, &rstatus)):         
            if not callback(mpgedit_edit_frames(self.ctx), 
                            mpgedit_edit_sec(self.ctx), 
                            mpgedit_edit_offset(self.ctx)): break
            
        if rstatus != 0:
            raise MpgeditError(rstatus, edit_errors[rstatus])
                        
    def frames(self):      
        return mpgedit_edit_frames(self.ctx)
        
    def sec(self):           
        return mpgedit_edit_sec(self.ctx)
        
    def offset(self):        
        return mpgedit_edit_offset(self.ctx)
                
    def info(self):
        return mpgedit_edit_frames(self.ctx), mpgedit_edit_sec(self.ctx), \
               mpgedit_edit_offset(self.ctx)
               
    def xing_header(self):
        cdef char *buf          
        header_dict = {}        
        if mpgedit_edit_has_xing(self.ctx):
            buf = <char*>malloc(1024)      
            xingheader2str(mpgedit_edit_xing_header(self.ctx), buf)
            header = buf
            free(buf)

            for line in header.splitlines():
                n, v = line.split('=')
                header_dict[n.strip()] = int(v.strip())        
        return header_dict        
                       
